@@ -7,12 +7,13 @@ use crate::request::{Fwd, Request};
 use core::num::{NonZeroU32, Wrapping};
 
 use std::borrow::Cow;
+use std::os::unix::io::RawFd;
 use std::path::Path;
 
 use serde::Deserialize;
 use ssh_format::Transformer;
 
-use std::os::unix::io::RawFd;
+use tokio_io_utility::read_exact_to_vec;
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash)]
 pub enum ForwardType {
@@ -38,16 +39,24 @@ impl Connection {
         Ok(())
     }
 
+    fn deserialize<'a, T: Deserialize<'a>>(&'a self) -> Result<T> {
+        // Ignore any trailing bytes to be forward compatible
+        Ok(self.transformer.deserialize()?.0)
+    }
+
     async fn read_and_deserialize<'a, T>(&'a mut self, size: usize) -> Result<T>
     where
         T: Deserialize<'a>,
     {
-        self.transformer.get_buffer().resize(size, 0);
-        self.raw_conn
-            .read(&mut self.transformer.get_buffer())
-            .await?;
-        // Ignore any trailing bytes to be forward compatible
-        Ok(self.transformer.deserialize()?.0)
+        self.transformer.get_buffer().clear();
+        read_exact_to_vec(
+            &mut self.raw_conn.stream,
+            self.transformer.get_buffer(),
+            size,
+        )
+        .await?;
+
+        self.deserialize()
     }
 
     /// Return size of the response.
@@ -68,7 +77,7 @@ impl Connection {
 
         // Ignore any trailing bytes to be forward compatible
         match self.raw_conn.try_read(self.transformer.get_buffer())? {
-            Some(_) => Ok(Some(self.transformer.deserialize()?.0)),
+            Some(_) => Ok(Some(self.deserialize()?)),
             None => Ok(None),
         }
     }
@@ -395,7 +404,7 @@ impl Connection {
     /// and remove its listener socket.
     ///
     /// **Only suitable to use in `Drop::drop`.**
-    pub fn close(self) -> Result<()> {
+    pub fn request_stop_listening_sync(self) -> Result<()> {
         shutdown_mux_master_from(self.raw_conn.into_std()?, self.transformer)
     }
 }
